package no.ntnu.newsrec.mapreduce;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.mapred.FileInputFormat;
import org.apache.hadoop.mapred.FileOutputFormat;
import org.apache.hadoop.mapred.JobClient;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;
import org.apache.hadoop.mapred.TextInputFormat;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * @author johneirik
 *
 * Job that calculates the category weights without time degrading functions
 *
 */
public class CategoryVector {

	public static class CategoryVectorMapper extends MapReduceBase
	implements Mapper<LongWritable, Text, Text, MapWritable> {

		private Text word = new Text();
		private DoubleWritable weight;
		private MapWritable mw = new MapWritable();


		public void map(LongWritable key, Text val,
				OutputCollector<Text, MapWritable> output, Reporter reporter)
						throws IOException {

			String line = new String(val.getBytes(), "UTF-8");
			StringTokenizer itr = new StringTokenizer(line, "\n");
			
			Constants.init();

			JSONObject jo;
			JSONArray categories;

			while (itr.hasMoreTokens()) {
				try {
					String next = itr.nextToken();
					jo = new JSONObject(next);
					word.set(jo.getString("userId"));
					
					String userAct = jo.getString("eventType");
					Double actWeight = Constants.USERACTMAP.get(userAct);
					weight = new DoubleWritable(actWeight);
					
					categories = jo.getJSONArray("categories");

					for(int i = 0; i < categories.length(); i++){
						mw.put(new Text(categories.getString(i)), weight);
					}

					output.collect(word, mw);
				} catch (JSONException e) {
					e.printStackTrace();
				} finally {
					word.clear();
					mw.clear();
				}
			}
		}
	}



	public static class CategoryVectorReducer extends MapReduceBase
	implements Reducer<Text, MapWritable, Text, Text> {


		public void reduce(Text key, Iterator<MapWritable> values,
				OutputCollector<Text, Text> output, Reporter reporter)
						throws IOException {

			Map<String, Double> mw = new HashMap<String, Double>();

			while (values.hasNext()) {
				MapWritable value = values.next();
				Iterator<Writable> keyIterator = value.keySet().iterator();

				while(keyIterator.hasNext()){
					Writable categoryKey = keyIterator.next();
					double temp;

					if(mw.containsKey(categoryKey.toString())){
						temp = mw.get(categoryKey.toString());
						temp += ((DoubleWritable)value.get(categoryKey)).get();
						mw.put(categoryKey.toString(), temp);
					}
					else{
						temp = ((DoubleWritable)value.get(categoryKey)).get();
						mw.put(categoryKey.toString(), temp);
					}
				}
				
			}
			
			// Code for storing the vectors through the REST API
			DefaultHttpClient httpClient = new DefaultHttpClient();
            HttpPost postRequest = new HttpPost(
                    "http://vm-6120.idi.ntnu.no:8080/news-rec-service/postCategoryInterests");

            String inputString = new String("{\"userId\":\"" + key + "\",\"interestWeights\":{");
            
            Iterator<String> keyIter = mw.keySet().iterator();
            
            while(keyIter.hasNext()){
            	String nextKey = keyIter.next();
            	inputString += "\"" + nextKey + "\":" + mw.get(nextKey);
            	if(keyIter.hasNext()){
            		inputString += ",";
            	}
            }
            
            inputString += "}}";
            		
            StringEntity input = new StringEntity(inputString);
            
            input.setContentType("application/json");
            postRequest.setEntity(input);

            HttpResponse response = httpClient.execute(postRequest);

            if (response.getStatusLine().getStatusCode() != 201) {
                throw new RuntimeException("Failed : HTTP error code : "
                        + response.getStatusLine().getStatusCode());
            }

            httpClient.getConnectionManager().shutdown();
			
			output.collect(key, new Text(mw.toString()));
			mw.clear();
		}
	}

	public static void main(String[] args) {
		JobClient client = new JobClient();
		JobConf conf = new JobConf(CategoryVector.class);

		conf.setJobName("CategoryVector");
		
		conf.setInputFormat(TextInputFormat.class);

		conf.setMapOutputKeyClass(Text.class);
		conf.setMapOutputValueClass(MapWritable.class);

		conf.setOutputKeyClass(Text.class);
		conf.setOutputValueClass(Text.class);
		
		FileSystem fs;
		try {
			fs = FileSystem.get(new Configuration());
			fs.delete(new Path("categoryvector"), true);
		} catch (IOException e2) {
			e2.printStackTrace();
		}

		FileInputFormat.addInputPath(conf, new Path("logdata/old"));
		FileOutputFormat.setOutputPath(conf, new Path("categoryvector"));

		conf.setMapperClass(CategoryVectorMapper.class);
		conf.setReducerClass(CategoryVectorReducer.class);

		client.setConf(conf);

		try {
			JobClient.runJob(conf);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}