package no.ntnu.newsrec.mapreduce;

import java.io.IOException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;

import no.ntnu.newsrec.upmongoreader.MongoReader;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.mapred.FileInputFormat;
import org.apache.hadoop.mapred.FileOutputFormat;
import org.apache.hadoop.mapred.JobClient;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;
import org.apache.hadoop.mapred.TextInputFormat;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.mongodb.BasicDBObject;
import com.mongodb.DBCollection;
import com.mongodb.DBObject;

/**
 * @author johneirik
 *
 * Job that calculates the category weights with a time degrading function
 *
 */
public class CategoryVectorV3 {

	public static class CategoryVectorV3Mapper extends MapReduceBase
	implements Mapper<LongWritable, Text, Text, MapWritable> {

		private Text userKey = new Text();
		private DoubleWritable weight;
		private MapWritable categoryWeightMap = new MapWritable();

		Calendar todayCal = Calendar.getInstance();


		public void map(LongWritable key, Text val,
				OutputCollector<Text, MapWritable> output, Reporter reporter)
						throws IOException {

			String inputText = new String(val.getBytes(), "UTF-8");
			StringTokenizer lines = new StringTokenizer(inputText, "\n");

			Constants.init();

			JSONObject userActObject;
			JSONArray userActCategories;

			while (lines.hasMoreTokens()) {
				try {

					userActObject = new JSONObject(lines.nextToken());

					String userId = userActObject.getString("userId");
					userKey.set(userId);

					String userActType = userActObject.getString("eventType");

					// Very messy code for handling timed events
					boolean timedAct = false;
					String fileName = "";

					if(userActType.equals("TIME_SPENT_PREVIEW")){
						timedAct = true;
						fileName = "articletime";
					} else if(userActType.equals("TIME_SPENT_ARTICLE_VIEW")){
						timedAct = true;
						fileName = "previewtime";
					}

					double userActTypeWeight = Constants.USERACTMAP.get(userActType);
					double thisActWeight = 0.0;


					// Gathering userstats from the HDFS, 
					// should rather allow storing mean times in database for easy lookup
					if(timedAct){
						//Initialize new default Hadoop Configuration
						Configuration conf = new Configuration();
						//Initialize new abstract Hadoop FileSystem
						FileSystem fs = FileSystem.get(conf);
						//Specify File Path of Hadoop File System
						Path filenamePath = new Path("userstats/" + fileName + "/part-00000");

						FSDataInputStream in = fs.open(filenamePath);
						String userStatLine = ""; 

						double actTime = 0.0;

						while(true){
							try{
								userStatLine = in.readUTF();
								String[] lineSplit = userStatLine.split("\t");
								if(lineSplit[0].equals(userId)){
									actTime = Double.parseDouble(lineSplit[1]);
								}
							}catch(IOException e){
								break;
							}
						}
						in.close();

						JSONObject properties = userActObject.getJSONObject("properties");
						double duration = Double.parseDouble(properties.getString("duration"));
						if(actTime < duration && actTime > 0.0) {
							thisActWeight = userActTypeWeight;
						}

					}
					else{
						thisActWeight = userActTypeWeight;
					}

					if(thisActWeight > 0.0){
						weight = new DoubleWritable(thisActWeight);

						userActCategories = userActObject.getJSONArray("categories");

						for(int i = 0; i < userActCategories.length(); i++){
							categoryWeightMap.put(new Text(userActCategories.getString(i)), weight);
						}		
						output.collect(userKey, categoryWeightMap);
					}
				} 

				catch (JSONException e) {
					e.printStackTrace();
				} 

				finally {
					userKey.clear();
					categoryWeightMap.clear();
				}
			}
		}
	}



	public static class CategoryVectorV3Reducer extends MapReduceBase
	implements Reducer<Text, MapWritable, Text, Text> {


		public void reduce(Text key, Iterator<MapWritable> values,
				OutputCollector<Text, Text> output, Reporter reporter)
						throws IOException {

			Map<String, Double> categoryVector = new HashMap<String, Double>();

			while (values.hasNext()) {
				MapWritable value = values.next();
				Iterator<Writable> keyIterator = value.keySet().iterator();

				while(keyIterator.hasNext()){
					Writable categoryKey = keyIterator.next();
					double temporaryTotalCategoryWeight;

					if(categoryVector.containsKey(categoryKey.toString())){
						temporaryTotalCategoryWeight = categoryVector.get(categoryKey.toString());
						temporaryTotalCategoryWeight += ((DoubleWritable)value.get(categoryKey)).get();
						categoryVector.put(categoryKey.toString(), temporaryTotalCategoryWeight);
					}
					else{
						temporaryTotalCategoryWeight = ((DoubleWritable)value.get(categoryKey)).get();
						categoryVector.put(categoryKey.toString(), temporaryTotalCategoryWeight);
					}
				}
			}

			MongoReader mr = new MongoReader();
			DBCollection collection = mr.getCollection("user-profiles");

			DBObject userProfile = collection.findOne(new BasicDBObject("_id", "719bca0668f2348359117df5b07c73c3"));

			DBObject oldCategories = (DBObject)userProfile.get("categoryWeights");

			Iterator<String> categoryIter = oldCategories.keySet().iterator();
			
			while(categoryIter.hasNext()){
				String category = categoryIter.next();
				double oldWeight = Double.parseDouble(oldCategories.get(category).toString())*Constants.DEGRADINGCONSTANT;
				
				if(categoryVector.containsKey(category)){
					Double newWeight = categoryVector.remove(category);
					newWeight += oldWeight;
					categoryVector.put(category, newWeight);
				}
				else{
					categoryVector.put(category, oldWeight);
				}
			}

			double highestTotalWeight = 0.0;
			Iterator<String> vectorIter = categoryVector.keySet().iterator();
			while(vectorIter.hasNext()){
				String category = vectorIter.next();
				if(highestTotalWeight < categoryVector.get(category)){
					highestTotalWeight = categoryVector.get(category);
				}
			}

			// Code for storing vectors though REST API
			DefaultHttpClient httpClient = new DefaultHttpClient();
			HttpPost postRequest = new HttpPost(
					"http://vm-6120.idi.ntnu.no:8080/news-rec-service/postCategoryInterests");


			String inputString = new String("{\"userId\":\"" + key + "\",\"interestWeights\":{");

			Iterator<String> categoryIterator = categoryVector.keySet().iterator();

			highestTotalWeight = highestTotalWeight / 100.0;

			while(categoryIterator.hasNext()){
				String nextKey = categoryIterator.next();
				inputString += "\"" + nextKey + "\":" + categoryVector.get(nextKey);
				if(categoryIterator.hasNext()){
					inputString += ",";
				}
				categoryVector.put(nextKey, categoryVector.get(nextKey)/highestTotalWeight);
			}

			inputString += "}}";

			StringEntity input = new StringEntity(inputString);

			input.setContentType("application/json");
			postRequest.setEntity(input);

			HttpResponse response = httpClient.execute(postRequest);

			if (response.getStatusLine().getStatusCode() != 201) {
				throw new RuntimeException("Failed : HTTP error code : "
						+ response.getStatusLine().getStatusCode());
			}

			httpClient.getConnectionManager().shutdown();

			output.collect(key, new Text(categoryVector.toString()));
			categoryVector.clear();
		}
	}

	public static void main(String[] args) {
		JobClient client = new JobClient();
		JobConf conf = new JobConf(CategoryVectorV3.class);

		conf.setJobName("CategoryVector3");

		conf.setInputFormat(TextInputFormat.class);

		conf.setMapOutputKeyClass(Text.class);
		conf.setMapOutputValueClass(MapWritable.class);

		conf.setOutputKeyClass(Text.class);
		conf.setOutputValueClass(Text.class);

		FileSystem fs;
		try {
			fs = FileSystem.get(new Configuration());
			fs.delete(new Path("categoryvector3"), true);
		} catch (IOException e2) {
			e2.printStackTrace();
		}

		FileInputFormat.addInputPath(conf, new Path("logdata/old/main.json"));
		FileOutputFormat.setOutputPath(conf, new Path("categoryvector3"));

		conf.setMapperClass(CategoryVectorV3Mapper.class);
		conf.setReducerClass(CategoryVectorV3Reducer.class);

		client.setConf(conf);

		try {
			JobClient.runJob(conf);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}