package no.ntnu.newsrec.mapreduce;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.MapWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.mapred.FileInputFormat;
import org.apache.hadoop.mapred.FileOutputFormat;
import org.apache.hadoop.mapred.JobClient;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;
import org.apache.hadoop.mapred.TextInputFormat;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * @author johneirik
 *
 * Job that calculates the keyword weights without time degrading functions
 *
 */
public class KeywordVector {

	public static class KeywordVectorMapper extends MapReduceBase
	implements Mapper<LongWritable, Text, Text, MapWritable> {

		private Text word = new Text();
		private  DoubleWritable weight;
		private MapWritable mw = new MapWritable();


		public void map(LongWritable key, Text val,
				OutputCollector<Text, MapWritable> output, Reporter reporter)
						throws IOException {

			String line = val.toString();
			StringTokenizer itr = new StringTokenizer(line, "\n");

			Constants.init();

			JSONObject jo;
			JSONArray keywords;

			while (itr.hasMoreTokens()) {
				try {
					String next = itr.nextToken();
					jo = new JSONObject(next);
					word.set(jo.getString("userId"));

					String userAct = jo.getString("eventType");
					Double actWeight = Constants.USERACTMAP.get(userAct);
					weight = new DoubleWritable(actWeight);

					keywords = jo.getJSONArray("tags");

					for(int i = 0; i < keywords.length(); i++){
						mw.put(new Text(keywords.getString(i)), weight);
					}

					output.collect(word, mw);
				} catch (JSONException e) {
					e.printStackTrace();
				} finally {
					word.clear();
					mw.clear();
				}
			}
		}
	}



	public static class KeywordVectorReducer extends MapReduceBase
	implements Reducer<Text, MapWritable, Text, Text> {


		public void reduce(Text key, Iterator<MapWritable> values,
				OutputCollector<Text, Text> output, Reporter reporter)
						throws IOException {

			HashMap<String, Double> mw = new HashMap<String, Double>();

			// Summarizing the weights
			while (values.hasNext()) {
				MapWritable value = values.next();
				Iterator<Writable> keyIterator = value.keySet().iterator();

				while(keyIterator.hasNext()){
					Writable keywordKey = keyIterator.next();
					double temp;

					if(mw.containsKey(keywordKey)){
						temp = mw.get(keywordKey);
						temp += ((DoubleWritable)value.get(keywordKey)).get();
						mw.put(keywordKey.toString(), temp);
					}
					else{
						temp = ((DoubleWritable)value.get(keywordKey)).get();
						mw.put(keywordKey.toString(), temp);
					}
				}
			}

			// Code for storing the vectors through the REST API
			DefaultHttpClient httpClient = new DefaultHttpClient();
			HttpPost postRequest = new HttpPost(
					"http://vm-6120.idi.ntnu.no:8080/news-rec-service/postContentInterests");

			String inputString = new String("{\"userId\":\"" + key + "\",\"interestWeights\":{");

			Iterator<String> keyIter = mw.keySet().iterator();

			while(keyIter.hasNext()){
				String nextKey = keyIter.next();
				inputString += "\"" + nextKey + "\":" + mw.get(nextKey);
				if(keyIter.hasNext()){
					inputString += ",";
				}
			}

			inputString += "}}";

			StringEntity input = new StringEntity(inputString, "UTF-8");

			input.setContentType("application/json");
			postRequest.setEntity(input);

			HttpResponse response = httpClient.execute(postRequest);

			if (response.getStatusLine().getStatusCode() != 201) {
				throw new RuntimeException("Failed : HTTP error code : "
						+ response.getStatusLine().getStatusCode());
			}

			httpClient.getConnectionManager().shutdown();

			output.collect(key, new Text(mw.toString()));
			mw.clear();
		}
	}

	public static void main(String[] args) {
		JobClient client = new JobClient();
		JobConf conf = new JobConf(KeywordVector.class);

		conf.setJobName("KeywordVector");

		conf.setInputFormat(TextInputFormat.class);

		conf.setMapOutputKeyClass(Text.class);
		conf.setMapOutputValueClass(MapWritable.class);

		conf.setOutputKeyClass(Text.class);
		conf.setOutputValueClass(Text.class);

		FileSystem fs;
		try {
			fs = FileSystem.get(new Configuration());
			fs.delete(new Path("keywordvector"), true);
		} catch (IOException e2) {
			e2.printStackTrace();
		}

		FileInputFormat.addInputPath(conf, new Path("logdata"));
		FileOutputFormat.setOutputPath(conf, new Path("keywordvector"));

		conf.setMapperClass(KeywordVectorMapper.class);
		conf.setReducerClass(KeywordVectorReducer.class);

		client.setConf(conf);

		try {
			JobClient.runJob(conf);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}