package no.ntnu.newsrec.mapreduce;

import java.io.IOException;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.DoubleWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.FileInputFormat;
import org.apache.hadoop.mapred.FileOutputFormat;
import org.apache.hadoop.mapred.JobClient;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;
import org.apache.hadoop.mapred.TextInputFormat;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * @author johneirik
 * 
 * Class with MapReduce job for summarizing the user time spent in 
 * article previews on average
 *
 */
public class PreviewTimeSummary {

	public static class UserStatMapper extends MapReduceBase 
	implements Mapper<LongWritable, Text, Text, DoubleWritable> {

		private Text word = new Text();
		private DoubleWritable seconds;


		public void map(LongWritable key, Text val,
				OutputCollector<Text, DoubleWritable> output, Reporter reporter)
						throws IOException {

			String line = val.toString();
			StringTokenizer itr = new StringTokenizer(line, "\n");

			Constants.init();

			JSONObject jo;

			while (itr.hasMoreTokens()) {
				try {
					jo = new JSONObject(itr.nextToken());
					word.set(jo.getString("userId"));

					String userAct = jo.getString("eventType");

					if(userAct.equals(Constants.USERACT[1])){
						JSONObject duration = jo.getJSONObject("properties");
						seconds = new DoubleWritable(Double.parseDouble(duration.getString("duration")));
						output.collect(word, seconds);
					}
				} catch (JSONException e) {
					e.printStackTrace();
				} finally{
					word.clear();
				}
			}
		}
	}



	public static class UserStatReducer extends MapReduceBase 
	implements Reducer<Text, DoubleWritable, Text, DoubleWritable> {


		public void reduce(Text key, Iterator<DoubleWritable> values,
				OutputCollector<Text, DoubleWritable> output, Reporter reporter)
						throws IOException {

			double sum = 0;
			double count = 0;

			while (values.hasNext()) {
				DoubleWritable value = values.next();
				sum += value.get();
				count += 1;
			}

			if(sum > 0  && count > 0) {
				sum = sum / count;
				output.collect(key, new DoubleWritable(sum));
			}
		}
	}


	public static void main(String[] args) {
		JobClient client = new JobClient();
		JobConf conf = new JobConf(PreviewTimeSummary.class);

		conf.setJobName("Preview time sum per user");

		conf.setInputFormat(TextInputFormat.class);

		conf.setMapOutputKeyClass(Text.class);
		conf.setMapOutputValueClass(DoubleWritable.class);

		conf.setOutputKeyClass(Text.class);
		conf.setOutputValueClass(DoubleWritable.class);

		// configuration should contain reference to your namenode
		FileSystem fs;
		try {
			fs = FileSystem.get(new Configuration());
			fs.delete(new Path("userstats/previewtime"), true);
		} catch (IOException e2) {
			e2.printStackTrace();
		}

		FileInputFormat.addInputPath(conf, new Path("logdata/main.json"));
		FileOutputFormat.setOutputPath(conf, new Path("userstats/previewtime"));

		conf.setMapperClass(UserStatMapper.class);
		conf.setReducerClass(UserStatReducer.class);

		client.setConf(conf);

		try {
			JobClient.runJob(conf);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}