package no.ntnu.newsrec.upmongoreader;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FSDataOutputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;

import com.mongodb.BasicDBObject;
import com.mongodb.BasicDBObjectBuilder;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;


/**
 * @author johneirik
 *
 * Class that extracts new data from MongoDB to HDFS by using a timestamp.
 *
 */
public class MongoDailyExtractor {

	public static void main (String [] args) {
		try{
			//Initialize new default Hadoop Configuration
			Configuration conf = new Configuration();
			//Initialize new abstract Hadoop FileSystem
			FileSystem fs = FileSystem.get(conf);
			//Specify File Path of Hadoop File System
			Path filenamePath = new Path("logdata/new/new.json");

			MongoReader mr = new MongoReader();

			DBCollection collection = mr.getCollection("audit-log");

			BasicDBObject query = new BasicDBObject();

			Calendar cal = Calendar.getInstance();
			long outDate = cal.getTimeInMillis();
			Date toDate = cal.getTime();

			// Getting timestamp, needs to be initialized

			Path timestampPath = new Path("timestamp/timestamp.txt");
			FSDataInputStream in = fs.open(timestampPath);
			long timestamp = in.readLong();
			in.close();
			
			cal.setTimeInMillis(timestamp);

			Date fromDate = cal.getTime();

			// Querying MongoDB using a timerange from last timestamp to now
			query.put("timestamp", BasicDBObjectBuilder.start("$gte", fromDate).add("$lte", toDate).get());
			
			DBCursor cursor = collection.find(query).sort(new BasicDBObject("timestamp", -1));
			
			//Check if file doesn't exist
			if (fs.exists(filenamePath)) {
				// if file exist, remove file first
				fs.delete(filenamePath, true);
			}
			//Write elements to file
			FSDataOutputStream out = fs.create(filenamePath);
			while(cursor.hasNext()) {
				String c = cursor.next().toString() +"\n";
				out.writeBytes(c);
			}

			out.close();
			cursor.close();
			mr.close();
			
			if (fs.exists(timestampPath)) {
				// if file exist, remove file first
				fs.delete(timestampPath, true);
			}
			
			FSDataOutputStream tsOut = fs.create(timestampPath);
			tsOut.writeLong(outDate);
			tsOut.close();
		} catch (IOException ioe) {
			System.err.println("IOException during operation: " + ioe.toString());
			System.exit(1);
		}
	}
}
