package no.ntnu.newsrec.upmongoreader;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataOutputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;

import com.mongodb.BasicDBObject;
import com.mongodb.BasicDBObjectBuilder;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;


/**
 * @author johneirik
 * 
 *  Class that extract data from MongoDB to HDFS from a specified time
 *  till now.
 *
 */
public class MongoExtractor {

	public static void main (String [] args) throws IOException {
		//Initialize new default Hadoop Configuration
		Configuration conf = new Configuration();
		//Initialize new abstract Hadoop FileSystem
		FileSystem fs = FileSystem.get(conf);
		//Specify File Path of Hadoop File System
		Path filenamePath = new Path("/user/johneirik/logdata/old/main.json");

		MongoReader mr = new MongoReader();

		DBCollection collection = mr.getCollection("audit-log");
		
		BasicDBObject query = new BasicDBObject();
		
		Calendar cal = Calendar.getInstance();
		
		Date toDate = cal.getTime();
		cal.add(Calendar.DATE, -90);
		Date fromDate = cal.getTime();
		
		// Querying for events within a timerange
		query.put("timestamp", BasicDBObjectBuilder.start("$gte", fromDate).add("$lte", toDate).get());
		
		DBCursor cursor = collection.find(query).sort(new BasicDBObject("timestamp", -1));

		try {
			//Check if file doesn't exist
			if (fs.exists(filenamePath)) {
				// if file exist, remove file first
				fs.delete(filenamePath, true);
			}
			//Write Configuration to File
			FSDataOutputStream out = fs.create(filenamePath);
			while(cursor.hasNext()) {
				String c = cursor.next().toString() +"\n";
				out.writeBytes(c);
			}

			out.close();
			cursor.close();
			mr.close();
		} catch (IOException ioe) {
			System.err.println("IOException during operation: " + ioe.toString());
			System.exit(1);
		}
	}
}
